# -*- coding: utf-8 -*-
import os
from tools import settings
from tools.config import get_config_data, run_update
from tools.messages import notify
from tools.utils import run_command, get_app, confirma


def run_runapp(application, action, opt=None, arg=None):
    data = get_config_data()
    if not data:
        return False

    name = ""
    container_id = None
    if application:
        container_id, name = get_app(
            application=application,
            title="Build/Run da Aplicacao",
            data=data,
            stop=False if action == "exec" else True
        )
        if not container_id:
            return False

    if action == 'build' and settings.USE_ECR:
        run_command(
            get_stdout=False,
            command_list=[
                {
                    'command': "aws ecr get-login --region {region}".format(region=data['aws_region']),
                    'run_stdout': True
                }
            ]
        )

    if action == "exec":
        print("\n\033[1m\033[94mRodando comando '{}' em '{}'\033[0m".format(
            " ".join(arg), name))
        os.system(
            "docker {cmd} -ti {app}{arg}".format(
                cmd=action,
                app=container_id,
                arg=" {}".format(" ".join(arg)) if arg else "")
        )
    else:
        run_command(
            get_stdout=False,
            title="Rodar Comando Docker: {}".format(action.upper()),
            command_list=[
                {
                    'command': "cd {} && docker-compose stop && docker stop $(docker ps -q)".format(
                        data['docker_compose_path']),
                    'run_stdout': False
                }
            ]
        )
        os.system(
            "cd {folder} && docker-compose {cmd} {opt} {app}".format(
                folder=data['docker_compose_path'],
                cmd=action,
                app=name,
                opt=opt if opt else "")
        )
    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    if action == "run":
        os.system(
            "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
        )
    if action == "build":
        notify(msg=u"A operação de Build foi concluída")


def run_debug(application):
    data = get_config_data()
    if not data:
        return False
    # 1. Identifica o container
    container_id, name = get_app(
        application=application,
        title="Rodar em Modo Depuração",
        data=data)
    if not container_id:
        return False

    # 2. Parar e reiniciar o container com service ports
    # docker-compose stop $app
    # docker-compose run --service-ports $app
    os.system('cls' if os.name == 'nt' else 'clear')
    os.chdir(data['project_path'])
    run_command(
        title="Modo Depuração: {}".format(name),
        get_stdout=True,
        command_list=[
            {
                'command': "cd {} && docker-compose stop {}".format(data['docker_compose_path'], name),
                'run_stdout': False
            },
        ]
    )
    os.system(
        'cd {} && docker-compose run --service-ports {}\n'.format(data['docker_compose_path'], name)
    )

    print("Reiniciando o container...")
    run_command(command_list=[{'command': "cd {} && docker-compose up -d {}".format(
        data['docker_compose_path'], name), 'run_stdout': False}, ])

    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    os.system(
        "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
    )
    return False


def run_telnet(application, port):
    data = get_config_data()
    if not data:
        return False

    container_id, name = get_app(
        application=application,
        title="Rodar Telnet",
        data=data
    )

    if not container_id:
        return False

    os.chdir(data['project_path'])
    os.system(
        'docker exec -ti {} telnet 127.0.0.1 {}'.format(
            container_id, port
        )
    )

    return False


def run_bash(application):

    data = get_config_data()
    if not data:
        return False

    container_id, name = get_app(
        application=application,
        title="Rodar Bash",
        data=data
    )

    if not container_id:
        return False

    os.chdir(data['project_path'])
    os.system(
        'docker exec -ti {} /bin/bash'.format(
            container_id
        )
    )

    return False


def run_test(application, using, rds):
    data = get_config_data()
    if not data:
        return False

    container_id, name = get_app(
        application=application,
        title="Rodar Teste",
        data=data
    )

    if not container_id:
        return False

    os.chdir(data['project_path'])
    os.system('cls' if os.name == 'nt' else 'clear')
    # Parar o container
    print("Rodar Testes - {}".format(name))
    print("Reiniciando container...")
    run_command(
        get_stdout=True,
        command_list=[
            {
                'command': "cd {} && docker-compose stop {}".format(data['docker_compose_path'], name),
                'run_stdout': False
            },
        ]
    )

    # Rodar o container com o endereco do
    # Banco de dados selecionado
    if rds:
        host = settings.STAGE_DB
        port = settings.STAGE_PORT
    else:
        host = settings.LOCAL_DB
        port = settings.LOCAL_PORT

    # Encontrar o programa
    test_app = using if using else "unittest"
    if not using:
        ret_pip = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "cd {} && docker-compose run {} pip freeze".format(
                    data['docker_compose_path'], name),
                    'run_stdout': False
                }
            ]
        )
        for test in settings.TEST_PRIORITY:
            if "{}==".format(test) in ret_pip:
                test_app = test
                break

    # Rodar novo container
    # Para Unittest, Django, Pytest e Nose rodar via Docker-Compose
    if test_app != 'behave':
        new_container_id = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "cd {} && docker-compose run -d -e DATABASE_HOST={} -e DATABASE_PORT={} {}".format(
                        data['docker_compose_path'],
                        host,
                        port,
                        name),
                    'run_stdout': False},
            ])
    else:
        # Para behave é preciso abrir via docker run --privileged
        # carregando as variaveis de ambiente
        env_path = os.path.join(
        data['docker_compose_path'],
            'docker-base.env'
        )
        with open(env_path, 'r') as file:
            env_data = file.read().split('\n')

        args = []
        for line in env_data:
            if not line or line.startswith("#") or line.startswith(" ") or line.startswith("DATABASE") or line.startswith("DJANGO_SETTINGS_MODULE"):
                continue

            var_name = line.split('=')[0]
            var_value = line.split('=')[1:]
            if isinstance(var_value, list):
                var_value = ''.join(var_value)

            args.append('-e {}="{}" '.format(var_name, var_value))

        env_args = " ".join(args) if args else ""
        
        command = "docker run -d --privileged -e DATABASE_HOST={host} -e DATABASE_PORT={port} -e DJANGO_SETTINGS_MODULE=app.settings {args} lidocker_{name}".format(
                        host=host,
                        port=port,
                        args=env_args,
                        name=name)

     
        new_container_id = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': command,
                    'run_stdout': False},
            ])


    if new_container_id:

        new_container_id = new_container_id.replace("\n", "")

        database_path = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': "docker exec -ti {} printenv | grep DATABASE_HOST".format(new_container_id),
                    'run_stdout': False}])
        print("\033[93m\n************************************")
        print("Rodando testes com: {}".format(test_app.upper()))
        print("Usando banco de dados: {}".format(
            database_path.replace("\n", "").split("=")[1])
        )
        print("Usando a Porta: {}".format(port))
        print("************************************\n\033[0m")
        
        if test_app == "django":
            command = "python /opt/app/manage.py test"
        elif test_app == "nose":
            command = "nosetests --with-coverage --cover-package=app"
        elif test_app == "pytest":
            command = "pytest"
        elif test_app == "behave":
            command = "behave tests/acceptance"
        else:
            command = "python -m unittest discover -v -s /opt/app"

        os.system(
            'docker exec -ti {} {}'.format(
                new_container_id, command
            )
        )
    else:
        print("ERRO: Nenhum container encontrado")

    print("Reiniciando container...")
    os.system("docker stop {}".format(new_container_id))
    os.system(
        "cd {} && docker-compose up -d {}".format(data['docker_compose_path'], name))

    # Exclui container extra
    # docker rm $(docker ps -a | grep host_run |  awk '{print $1}')
    os.system(
        "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
    )
    notify(msg="Teste Unitário em {} finalizado.".format(name))
    return False


def rebuild_docker(no_confirm):
    data = get_config_data()
    if not data:
        return False

    if no_confirm:
        resp = "S"
    else:
        resp = confirma(
            u"Este comando exclui todas as imagens\n"
            u"e containers existentes na máquina,\n"
            u"e inicia um novo Update/Build.\n"
            u"\n\033[91mCertifique-se que você tenha um backup\n"
            u"do banco de dados antes de rodar esse comando e"
            u"que todas as alterações importantes estejam commitadas.\033[0m\n\n"
            u"Deseja continuar")

    if resp == "S":
        # Parar containers
        run_command(
            get_stdout=False,
            title=None,
            command_list=[
                {
                    'command': "cd {} && docker-compose stop && docker stop $(docker ps -q)".format(
                        data['docker_compose_path']),
                    'run_stdout': False
                }
            ]
        )
        # docker rm $(docker ps -a -q)
        run_command(
            title="Excluir Containers do Docker",
            command_list=[
                {
                    'command': 'docker rm $(docker ps -a -q)',
                    'run_stdout': False
                }
            ]
        )
        # docker rmi $(docker images -q)
        run_command(
            title="Excluir Imagens do Docker",
            command_list=[
                {
                    'command': 'docker rmi $(docker images -q)',
                    'run_stdout': False
                }
            ]
        )

        # Roda Update
        run_update(no_confirm=True, stable=True, staging=False)

        # Roda Build
        run_runapp(application=None, action="build")

        # Finaliza
        notify(msg="Rebuild dos Containers finalizado")
        os.system('cls' if os.name == 'nt' else 'clear')
        print(u"O Rebuild foi concluído.")
        print(u"Antes de iniciar os containers, digite o comando:")
        print(u"'cd {} && docker-compose up service.postgres.local'".format(data['docker_compose_path']))
        print(u"para iniciar o Banco de dados pela primeira vez.")
        print(u"Em seguida use o comando 'li run'.")
        return True



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
from tools.utils import bcolors

INICIAL = u"Este help mostra os comandos da ferramenta LI-Tools.\nPara apenas visualizar os comandos e suas opções, digite 'li'.\nPara ver a Ajuda de todos os comandos, digite 'li help'.\nPara ver a Ajuda de um comando específico digite 'li help <comando>'.\n"

HELP_COMMANDS = [{'command': 'config',
                  'options': 'li config [--clone-only]',
                  'description': u'Use para configurar o ambiente de desenvolvimento local.\nUse a opção `--clone-only` para clonar/baixar os repositórios apenas',
                  'examples': None,
                  'long_desc': None},
                 {'command': 'deploy',
                  'options': 'li deploy',
                  'description': u'Use para fazer o deploy de um ambiente local na Amazon.\nÉ preciso rodar este comando na pasta-raiz de um dos repositorios clonados pelo comando `li config`',
                  'examples': None,
                  'long_desc': None},
                 {'command': 'telnet',
                  'options': 'li telnet <app> <port>',
                  'description': u'Use para abrir a aplicação informada num TTY que faz um telnet dentro do Docker para o endereco 127.0.0.1 e a porta especificada.\nSe não informar a aplicação a ferramenta irá perguntar qual aplicação abrir, a partir dos containers em atividade.\nEquivale ao comando "docker exec -ti _container_ telnet 127.0.0.1 _porta_"',
                  'examples': 'li telnet worker 6908',
                  'long_desc': None},
                 {'command': 'test',
                  'options': 'li test [<app>] [--using=(django|nose|pytest|behave)] [--rds]',
                  'description': u'Use para rodar os testes unitários da aplicação.\nA ferramenta permite o uso das livrarias: behave, nose (com coverage) e pytest. Para isto basta informar qual utilizar com a opção "--using".\nSe nenhuma livraria estiver instalada ou nenhuma for selecionada, será usado o Unittest padrão do Python.\nUtilize as opções abaixo para configurar os testes:\n* --using: Use uma das opções a seguir: nose, pytest, django, behave\n* --rds: Use o banco de dados na Amazon (utilizado pelo ambiente de Staging) ao invés do banco de dados local, para rodar os testes',
                  'examples': 'li test painel (abre o LI-AppPainel rodando unittest e banco de dados local)\nli test loja --using=django (abre a LI-AppLoja rodando django e banco de dados local)\nli test worker --using=nose --rds (abre o LI-Worker rodando nose e banco de dados de staging na Amazon)',
                  'long_desc': None},
                 {'command': 'bash',
                  'options': 'li bash <app>',
                  'description': u'Use para abrir a aplicação informada num terminal bash (usuario root).\nSe não informar a aplicação a ferramenta irá perguntar qual aplicação abrir, a partir dos containers em atividade.\nEquivale ao comando "docker exec -ti _container_ /bin/bash"',
                  'examples': 'li bash carrinho',
                  'long_desc': None},
                 {'command': 'run',
                  'options': 'li run <app>',
                  'description': u'Roda a aplicação informada e suas dependências ou todas as aplicações/containers se não informar nenhuma aplicação.\nEquivale ao comando "docker-compose stop && docker-compose up _aplicacao_"',
                  'examples': 'li run (roda todos os containers)\nli run painel (roda o container do Painel e os containers dependentes',
                  'long_desc': None},
                 {'command': 'build',
                  'options': 'li build <app> [--no-cache]',
                  'description': u'Use para fazer a build do container da aplicação selecionada. Se nenhuma aplicação for informada, será feito a build de todos os containers:\n* --no-cache: Durante a build, forçar o download de todas as livrarias/dependencias\nEquivale ao comando "docker-compose stop && docker-compose build _aplicacao_"',
                  'examples': 'li build (faz o build de todos os containers)\nli build loja (faz a build do container da Loja)',
                  'long_desc': None},
                 {'command': 'update',
                  'options': 'li update [--yes] [--production | --staging]',
                  'description': u'Roda em cada repositório clonado pelo comando `li config` o comando "git remote update && git fetch && git pull --all", desde que o repositorio esteja na branch prodution, staging, beta, release ou master (repositórios que estejam em outros branchs são ignorados)\nEm adição ele clona novos repositórios que não estejam na pasta do projeto informado.\nUse a opção --yes ou -y para confirmar automaticamente as operações.\nUse a opção --production para fazer o git checkout para a branch mais estável do projeto (exemplo: production, master, etc) antes do update.',
                  'examples': u'li update (atualiza todas as branchs atuais)\nli update --production (faz o checkout para a branch mais estável e atualiza)\nli update --staging (faz o checkout para staging/beta e atualiza os repositórios',
                  'long_desc': None},
                 {'command': 'list',
                  'options': 'li list [<libs>...]',
                  'description': u'Lista as aplicações existentes no Docker-Compose, e informa as seguintes informações:\n* O nome da aplicação e se o container está rodando\n* A branch em que o container está.\n* As diferenças no branch, em relação ao VCS (commits a frente/atrás, etc...).\n* A porta que está sendo exposta no container\n* A versão do LI-Repo que está instalado dentro do container.\n\nPara listar outras livrarias, digite elas após o comando, separando com espaço.',
                  'examples': 'li list (para a lista padrao)\nli list django flask gunicorn (Para listar as versões do Django, Flask e Gunicorn que estão rodando dentro dos containers.)',
                  'long_desc': None},
                 {'command': 'rebuild',
                  'options': 'li rebuild',
                  'description': u'Este comando exclui todas as imagens e containers existentes na máquina.\nEm seguida atualiza os repositórios existentes e baixa os novos.\nApós isso, inicia a build de todos os containers.\nCertifique-se de ter feito um backup do seu banco de dados local, antes de iniciar esse comando.',
                  'examples': None,
                  'long_desc': None},
                  {'command': 'tunnel',
                  'options': 'li tunnel <subdominio> <aplicacao>',
                  'description': u'Este comando cria um tunel reverso a partir de uma instância amazon (tunnel.awsli.com.br) para uma das aplicações rodando localmente.\nUse quando é necessário informar, em um serviço de terceiros, um endereço http ou https, ao invés de "localhost".\nO ngrok usa o <subdomínio> informado para criar a URL (subdominio.tunnel.awsli.com.br), tanto para http, quanto https e redireciona as chamadas para a porta da <aplicação> informada.',
                  'examples': u'li tunnel retorno pagador (Cria o tunel "http://retorno.tunnel.awsli.com.br" e redireciona as chamadas para a aplicação LI-Api-Pagador)\nli tunnel ret-facebook painel (Cria o túnel "http://ret-facebook.tunnel.awsli.com.br" e redireciona as requisições para a LI-AppPainel)',
                  'long_desc': None}
                  ]


def get_help(app=None):
    if app:
        command_help = [
            obj
            for obj in HELP_COMMANDS
            if obj.get('command') == app
        ]
        if command_help:
            show_help(command_help, show_long_desc=True)
            return True
        else:
            return True
    else:
        show_help(sorted(HELP_COMMANDS, key=lambda x: x['command']))
        return True


def show_help(help_list, show_long_desc=False):
    if not show_long_desc:
        print(INICIAL)
    for command in help_list:
        print("\n{}{}Commando: {}{}".format(
            bcolors.BOLD,
            bcolors.WARNING,
            command['command'].upper(),
            bcolors.ENDC
        )
        )
        if command['options']:
            print("{}Opções:{}".format(bcolors.BOLD, bcolors.ENDC))
            print(command['options'])
        print("\n{}Resumo:{}".format(bcolors.BOLD, bcolors.ENDC))
        print(command['description'].encode("UTF-8"))
        if show_long_desc and command['long_desc']:
            print('\n{}'.format(command['long_desc']))
        if command['examples']:
            print("\n{}Exemplos:{}".format(bcolors.BOLD, bcolors.ENDC))
            print(command['examples'])



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import json
import urllib2
from distutils.version import StrictVersion, LooseVersion
from tools import __version__
from tools import settings


def versions():
    url = "https://pypi.python.org/pypi/{}/json".format(settings.PYPI_NAME)
    data = None
    versions = None
    try:
        ret = urllib2.urlopen(urllib2.Request(url), timeout=1)
        data = json.load(ret)
    except:
        pass
    if data:
        versions = data["releases"].keys()
        versions.sort(key=LooseVersion)
    return versions


def show_version_warning():
    last_version = __version__
    version_data = versions()
    if version_data:
        last_version = version_data[-1]
    if LooseVersion(last_version) > LooseVersion(__version__) and \
        "rc" not in last_version:
        print("\033[91mSua versão está desatualizada.")
        print("Última versão: {}\n\033[0m".format(last_version))



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import subprocess
import sys
from unidecode import unidecode

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def confirma(pergunta):
    """Retorna S ou N"""
    resposta_ok = False
    while not resposta_ok:
        resposta = raw_input(u"\n{} (s/n)? ".format(pergunta).encode("UTF-8"))
        if resposta and resposta[0].upper() in ["S", "N"]:
            resposta_ok = True
    return resposta[0].upper()


def unitext(text):
    if type(text) == str:
        text = unicode(text, 'utf-8')
    text = unidecode(text)
    return text


def run_command(command_list, title=None, get_stdout=False):
    if title:
        print(u"\033[1m\033[93m\n\n>> {}".format(unitext(title)))
        print(u"{:*^{num}}\033[0m".format(
            '',
            num=len(title) + 3)
        )
    try:
        for task in command_list:
            if task['run_stdout']:
                command = subprocess.check_output(
                    task['command'],
                    shell=True
                )

                if not command:
                    print('Ocorreu um erro. Processo abortado')
                    return False

                ret = subprocess.call(
                    command,
                    shell=True
                )
            elif get_stdout is True:
                ret = subprocess.check_output(
                    task['command'],
                    shell=True
                )
            else:
                ret = subprocess.call(
                    task['command'],
                    shell=True,
                    stderr=subprocess.STDOUT
                )

            if ret != 0 and not get_stdout:
                print('Ocorreu um erro. Processo abortado')
                return False
    except:
        return False

    return True if not get_stdout else ret


def get_app(application, data, title=None, stop=False):
    # 1. Lista todos os containers que estao rodando
    # docker ps -a | grep painel | awk '{print $1,$2}'
    if not stop:
        ret = run_command(
            title=title,
            get_stdout=True,
            command_list=[
                {
                    'command': "docker ps | awk '{print $1, $NF}'",
                    'run_stdout': False
                }
            ]
        )
    else:
        ret = run_command(
            title=title,
            get_stdout=True,
            command_list=[
                {
                    'command': "docker ps -a | awk '{print $1, $NF}'",
                    'run_stdout': False
                }
            ]
        )
    raw_list = ret.split('\n')

    app_list = []

    for obj in raw_list:
        if obj.startswith("CONTAINER"):
            continue
        if len(obj.split(" ")) != 2:
            continue

        app_list.append((
            obj.split(" ")[0],
            obj.split(" ")[1]
        ))

    # 2. Identifica qual o container que bate com o app solicitado
    filtered_list = [
        app
        for app in app_list
        if application and application in app[1]
    ]

    ask_for_app = False
    if filtered_list:
        if len(filtered_list) == 1:
            return (filtered_list[0][0], filtered_list[0][1])
        else:
            ask_for_app = True
    elif app_list:
        ask_for_app = True
    else:
        print("Nenhum aplicativo encontrado.")
        return (None, None)

    if ask_for_app:
        all_apps = filtered_list or app_list
        i = 1
        for app in all_apps:
            print("{}. {}".format(i, app[1]))
            i += 1
        resposta_ok = False
        print("\n")
        while not resposta_ok:
            try:
                rep = raw_input(
                    "Selecione o App: (1-{}): ".format(i - 1))
                if rep and int(rep) in xrange(1, i):
                    resposta_ok = True
            except KeyboardInterrupt:
                print("\n")
                return (None, None)
            except:
                pass
        return (all_apps[int(rep) - 1][0], all_apps[int(rep) - 1][1])

def progress_bar(iteration, total, prefix='Lendo',
                 suffix='Complete', barLength=50):
    """
    Gerador de Barra de Progresso
    """
    formatStr = "{0:.2f}"
    percents = formatStr.format(100 * (iteration / float(total)))
    filledLength = int(round(barLength * iteration / float(total)))
    bar = '█' * filledLength + '-' * (barLength - filledLength)
    sys.stdout.write(
        '\r%s |%s| %s%s %s ' %
        (prefix, bar, percents, '%', suffix)),
    sys.stdout.flush()




# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import datadog
import platform
import requests
import slackweb
from tools import settings
from tools.utils import run_command, unitext


class Message():
    def __init__(
            self,
            config,
            branch,
            commit,
            repo,
            action,
            test=False,
            *args,
            **kwargs):
        self.config = config
        self.branch = branch
        self.commit = commit
        self.repo = repo
        self.test = test
        self.action = action
        print("\033[1m\033[93m\n>> Enviando mensagens")
        print("*********************\033[0m")

    def send(self, alert_type="info"):
        if settings.USE_DATADOG:
            self.send_datadog(alert_type)
        if settings.USE_SLACK:
            self.send_slack()
        if settings.USE_GRAFANA:
            self.send_grafana()

    def send_datadog(self, alert_type):

        options = {
            'api_key': self.config['datadog_api_key'],
            'app_key': self.config['datadog_app_key']
        }

        title = "DEPLOY {}: {}/{}".format(
            self.action, self.repo, self.branch.name)
        text = self.commit
        tags = ["deploy"]

        if not self.test:
            datadog.initialize(**options)
            datadog.api.Event.create(
                title=title,
                text=text,
                tags=tags,
                alert_type=alert_type if not self.test else "info"
            )
            print("Datadog OK")
        else:
            return "{}\n{}".format(title, text)

    def send_slack(self):

        text = "DEPLOY {}: {}/{}\n{}".format(
            self.action,
            self.repo,
            self.branch.name,
            self.commit
        )

        if not self.test:
            slack = slackweb.Slack(
                url=self.config['slack_url']
            )

            slack.notify(
                text=text,
                channel="#{}".format(
                    self.config['slack_channel']) if not self.test else settings.TEST_CHANNEL,
                username=self.config['slack_user'],
                icon_emoji=":{}:".format(
                    self.config['slack_icon']))
            print("Slack OK")
        else:
            return text

    def send_grafana(self):
        if self.action == "INICIADO":
            status = "started"
            value = 1
        else:
            status = "ended"
            value = 0
        
        post_data="li_aws_deploy,action=deploy,user={},repo={},status={},env={} value={}".format(
            self.config['slack_user'],
            self.repo,
            status,
            self.branch,
            value
        )
        if not self.test:
            try:
                ret = requests.post(
                settings.GRAFANA_MSG_URL,
                data=post_data,
                headers={"Content-Type": "application/x-www-form-urlencoded"},
                timeout=1
                )
                if ret.status_code == requests.codes.ok:
                    print("Grafana OK")
                else:
                    print("Ocorreu um erro ao tentar enviar mensagem ao Grafana")
            except:
                print("Ocorreu um erro ao tentar enviar mensagem ao Grafana")
        else:
            return "Passou pelo Grafana"


def notify(msg, title=None):
    if not title:
        title = "{}-Tools".format(settings.TERMINAL_CMD.upper())
    if platform.system().lower() == 'linux':  # Linux
        run_command(
            command_list=[
                    {
                        'command': 'notify-send -i gsd-xrandr {title} {msg}'.format(
                            title=u'"{}"'.format(unitext(title)),
                            msg=u'"{}"'.format(unitext(msg))),
                        'run_stdout': False
                    }
                ],
            get_stdout=True,
            title=None)
    elif platform.system().lower() != "windows":  # Mac
        run_command(
            command_list=[
                {
                    'command': "osascript -e 'display notification {msg} with title {title}'".format(
                        title=u'"{}"'.format(unitext(title)),
                        msg=u'"{}"'.format(unitext(msg))),
                    'run_stdout': False}],
            get_stdout=False,
            title=None)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import unittest
import os
from tools.config import get_config_data
from tools.compress import minifyJS, minifyCSS
from tools.messages import Message
from tools.utils import run_command
from os.path import expanduser
from unittest.mock import Mock


class TestDeploy(unittest.TestCase):

    maxDiff = None

    def test_ler_arquivo_config(self):
        self.assertIsInstance(
            get_config_data(filename="test_tool"),
            dict
        )
        filepath_test = "{}/.test_tool".format(expanduser("~"))
        self.assertTrue(
            os.path.isfile(filepath_test)
        )

    def teste_compress_JS(self):
        response = 'console.log("Olá Mundo");function teste(){var a=1;if(a>0){console.log("número positivo")}else{console.log("número menor ou igual a zero");a=a-1}};teste();console.log("fim");'
        source = [
            'js/javascript_file01.js',
            'js/javascript_file02.js'
        ]

        minifyJS(
            baseDir="./test_files/",
            source=source
        )

        with open("./test_files/js/all.min.js") as test_file:
            line = test_file.read()

        self.assertEqual(
            line,
            response
        )

    def teste_compress_CSS(self):
        response = '.teste,#id01{display:flex;justify-content:flex-start}p{font-family:"Times New Roman",Georgia,Serif;color:rgba(255,255,128,10)}@media screen and (min-width:480px){body{background-color:lightgreen}}'
        source = [
            'css/css_file01.css',
            'css/css_file02.css'
        ]

        minifyCSS(
            baseDir="./test_files/",
            source=source
        )

        with open("./test_files/css/all.min.css") as test_file:
            line = test_file.read()

        self.assertEqual(
            line,
            response
        )

    def test_send_message_datadog(self):
        result = "DEPLOY TESTE: repositorio_teste/teste\nteste de envio"

        fake_branch = Mock()
        fake_branch.name = "teste"

        config = get_config_data()
        message = Message(
            config=config,
            branch=fake_branch,
            commit="teste de envio",
            repo="repositorio_teste",
            action="TESTE",
            test=True)

        self.assertEqual(
            message.send_datadog(),
            result
        )

    def test_send_message_slack(self):
        result = "DEPLOY TESTE: repositorio_teste/teste\nteste de envio"

        fake_branch = Mock()
        fake_branch.name = "teste"

        config = get_config_data()
        message = Message(
            config=config,
            branch=fake_branch,
            commit="teste de envio",
            repo="repositorio_teste",
            action="TESTE",
            test=True)

        self.assertEqual(
            message.send_slack(),
            result
        )

    def test_run_command(self):
        self.assertTrue(
            run_command(
                title="TESTE",
                command_list=[
                    {
                        'command': "ls",
                        'run_stdout': False
                    }
                ]
            )
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import yaml
import os
import re
from li_tabulate import tabulate
from git import Repo
from tools.config import get_config_data
from tools.utils import run_command, bcolors, progress_bar
from tools.settings import DOCKER_PATH_VAR, LIBRARIES


def show_list(libs=[]):
    data = get_config_data()
    if not data:
        return False

    dc_path = os.path.join(
        data['docker_compose_path'],
        'docker-compose.yml'
    )

    with open(dc_path, 'r') as file:
        dc_data = yaml.load(file)

    print("\n\033[93m>> Lendo Aplicações Docker")
    print("**************************\033[0m\n")

    services_list = dc_data['services']

    table_data = []
    total = len(services_list)
    i = 0
    blank = max([
        len(obj)
        for obj in services_list
    ])
    for service in sorted(services_list):
        # Barra de progresso
        i += 1
        progress_bar(i, total, suffix=service.ljust(blank))
        # 1. Checa se o container está rodando
        docker_ret = run_command(
            command_list=[
                {
                    'command': "docker ps | grep {service}".format(
                        service=service),
                    'run_stdout': False
                }
            ],
            get_stdout=True,
            title=False
        )
        if docker_ret:
            container = docker_ret[:12]
        else:
            container = None

        if container:
            rodando = "{}SIM{}".format(bcolors.OKGREEN, bcolors.ENDC)
        else:
            rodando = "{}NÃO{}".format(bcolors.FAIL, bcolors.ENDC)

        # 2. Checa a branch
        try:
            caminho_dict = services_list.get(service)
            path = caminho_dict.get('build').get('context')
            caminho_path = os.path.join(
                data['project_path'],
                *path.split("/")[1:]
            )
            repo = Repo(caminho_path)
            branch = repo.active_branch.name
        except:
            branch = "--"
            caminho_path = None

        # 3. Checa status do Git
        if branch != "--":
            os.chdir(caminho_path)
            status_ret = run_command(
                command_list=[
                    {
                        'command': 'git status -bs --porcelain',
                        'run_stdout': False
                    }
                ],
                get_stdout=True,
                title=None
            )
            if status_ret.count('\n') > 1 or "[" in status_ret:
                branch = "{}{}{}".format(bcolors.WARNING, branch, bcolors.ENDC)
                ahead = re.search("ahead (\d+)", status_ret)
                behind = re.search("behind (\d+)", status_ret)
                if ahead or behind:
                    branch = "{} {}[{}{}{}]{}".format(
                        branch,
                        bcolors.WARNING,
                        "{}+{}{}".format(bcolors.OKBLUE, ahead.group(1), bcolors.ENDC) if ahead else "",
                        "{}-{}{}".format(bcolors.FAIL, behind.group(1), bcolors.ENDC) if behind else "",
                        bcolors.WARNING,
                        bcolors.ENDC
                    )

        # 4. Checa versão das livrarias
        lib_list = []
        pip_ret = None
        if caminho_path:
            for lib in LIBRARIES + libs:
                if container:
                    pip_ret = run_command(
                        command_list=[
                            {
                                'command': 'docker exec -ti {container} pip freeze | grep -i {library}== | tail -1'.format(
                                    container=container,
                                    library=lib),
                                'run_stdout': False}],
                        get_stdout=True,
                        title=None)
                elif branch != "--":
                    try:
                        pip_ret = run_command(
                            command_list=[
                                {
                                    'command': 'cd {path} && docker-compose run {service} pip freeze | grep -i {library}== | tail -1'.format(
                                        path=data['docker_compose_path'],
                                        service=service,
                                        library=lib
                                    ),
                                    'run_stdout':False
                                }
                            ],
                            get_stdout=True,
                            title=None
                        )
                    except:
                        pip_ret = None
                if pip_ret:
                    lib_list.append(pip_ret.split("==")[1])
                else:
                    lib_list.append("--" if "SIM" in rodando else "")

        # 5. Pega a Porta
        try:
            porta = dc_data['services'][service]['ports'][0].split(":")[0]
        except:
            porta = "--"
            
        table_data.append([service, branch, rodando, porta] + lib_list)

    # Exclui containers extra
    os.system(
        "docker rm $(docker ps -a | grep _run_ |  awk '{print $1}')"
    )
    os.system('cls' if os.name == 'nt' else 'clear')
    print("\n\033[93m>> Listar Aplicações Docker")
    print("**************************\033[0m\n")
    print(tabulate(
        table_data,
        headers=["Aplicação", "Branch", "Rodando", "Porta"] + LIBRARIES + libs
    )
    )



# -*- coding: utf-8 -*-
"""
Settings para o LI-Tools
"""
from __future__ import print_function, unicode_literals, with_statement, nested_scopes


# Linha de comando para o prompt
################################
TERMINAL_CMD = "li"

# Configurações do Projeto
##########################
CONFIG_FILE = "li-config"
ENV_NAME = "LI_PROJECT_PATH"

STAGE_DB = "db-staging.awsli.com.br"
STAGE_PORT = 5432

LOCAL_DB = 'service.postgres.local'
LOCAL_PORT = 5432

PYPI_NAME = "li-aws-deploy"

DOCKER_PATH_VAR = '\$\{LI_PROJECT_PATH\}'

CHECK_VPN = True

# Configurações para Testes
# Se não for informado no parametro
# rodar o teste usando o programa pela prioridade abaixo
# Se não encontrar nenhum destes, usa o unittest padrão
TEST_PRIORITY = [
    'behave',
    'nose',
    'pytest'
]

# Lista de Aplicações do Projeto
################################

# A ordem dos branchs é da
# branch mais volátil até a de producao
# Ex: develop, master
APPLICATIONS = [
    ('li-docker', ['master']),
    ('LI-Api-Carrinho', ['development', 'staging', 'production']),
    ('LI-Api-Catalogo', ['development', 'staging', 'production']),
    ('LI-Api-Envio', ['development', 'staging', 'production']),
    ('LI-Api-Faturamento', ['development', 'staging', 'production']),
    ('li-api-integration', ['development', 'staging', 'production']),
    ('LI-Api-Marketplace', ['development', 'staging', 'production']),
    ('LI-Api-Pagador', ['development', 'staging', 'production']),
    ('LI-Api-Pedido', ['development', 'staging', 'production']),
    ('LI-Api-Plataforma', ['development', 'staging', 'production']),
    ('LI-Api-V2', ['development', 'staging', 'production']),
    ('LI-AppApi', ['development', 'staging', 'production']),
    ('LI-AppConciliacao', ['staging', 'production']),
    ('LI-AppLoja', ['development', 'staging', 'production']),
    ('LI-AppPainel', ['development', 'staging', 'production']),
    ('LI-Worker', ['development', 'staging', 'production']),
    ('li-api-pagamento', ['beta', 'master']),
    ('li-worker-pagamento', ['beta', 'master']),
    ('li-worker-integration', ['development', 'staging', 'production']),
    ('LI-Repo', ['master']),
    ('LI-ApiClient', ['master']),
    ('LI-Common', ['master']),
    ('LI-Api-Flask', ['master']),
    ('Li-Worker-Importacao', ['staging', 'production']),
    ('LI-AWS-Deploy', ['master']),
    ('LI-OpsWorks', ['staging', 'production']),
    ('LI-Standalone', ['staging', 'production']),
    ('LI-Pagador', ['master']),
    ('LI-Pagador-Deposito', ['master']),
    ('LI-Pagador-MercadoPago', ['master']),
    ('LI-Pagador-Paghiper', ['master']),
    ('LI-Pagador-Boleto', ['master']),
    ('LI-Pagador-PagSeguro', ['master']),
    ('LI-Pagador-Entrega', ['master']),
    ('LI-Pagador-PayPal', ['master']),
    ('LI-Pagador-PagarMe', ['master']),
    ('LI-Pagador-PayPal-Transparente', ['master']),
    ('LI-Pagador-MercadoPago-Transparente', ['master']),
    ('li_testing', ['master', 'release']),
    ('li-internal-services', ['master'])
]

# Lista as Livrarias padrões do Projeto
# Os nomes abaixo devem ser o mesmo que aparece no comando 'pip freeze'
# Qto mais livrarias na lista, mais lento o comando 'li list' vai ficar
LIBRARIES = [
    'li-repo'
]

# Aplicações que minificam arquivos durante o deploy
MINIFY_BEFORE = [
    'LI-AppLoja'
]

# Aplicações que sincronizam arquivos no S3 durante o deploy
SYNC_S3 = [
    'LI-AppLoja',
    'LI-AppPainel'
]

# Compressão de arquivos estáticos
##################################
baseDirStatic = ["static", "loja", "estrutura", "v1"]

# Arquivos JS para minificar num unico arquivo
jsSources = [
    ("js", "jquery-1.10.1.min.js"),
    ("js", "jquery-ui.js"),
    ("js", "bootstrap.min.js"),
    ("js", "css3-mediaqueries.js"),
    ("js", "jquery.flexslider-min.js"),
    ("js", "jquery.mask.min.js"),
    ("js", "modernizr.custom.17475.js"),
    ("js", "jquery.cookie.min.js"),
    ("js", "jquery.rwdImageMaps.min.js"),
    ("js", "main.js")
]

# Arquivos CSS para minificar num unico arquivo
cssSources = [
    ("css", "bootstrap.css"),
    ("css", "font-awesome.css"),
    ("css", "font-awesome-ie7.css"),
    ("css", "font-awesome-v4.css"),
    ("css", "flexslider.css"),
    ("css", "prettify.css"),
    ("css", "es-cus.css"),
    ("css", "style.css"),
    ("css", "cores.css")
]

# Arquivos JS para minificar individualmente
jsAlone = [
    ("js", "produto.js"),
    ("js", "carrinho.js"),
    ("js", "checkout.js")
]

# Arquivos CSS para minificar individualmente
cssAlone = [
    ("css", "tema-escuro.css"),
    ("css", "ie-fix.css")
]

# CONFIGURACAO PARA DOCKER
##########################
DOCKER_REPO_NAME = "LI-Docker"
DOCKERFILE_NAME = "Dockerfile_local"

# CONFIGURAÇÃO PARA VCS
#######################
VCS_NAME = "BitBucket"
VCS_BASE_URL = "https://bitbucket.org/lojaintegradateam/"


# CONFIGURACAO PARA AMAZON WEB SERVICES
#######################################
USE_AWS = True
S3_SYNC_CMD = "aws s3 sync static/ s3://lojaintegrada.cdn/{branch}/static/ --acl public-read"

# AWS EC2 CONTAINER SERVICE
###########################

# Caso o nome do container seja
# diferente do nome da aplicacao
# fazer o de/para aqui
USE_ECR = True
ECR_NAME = {
    'LI-AppPainel': 'app-painel-production',
    'LI-AppApi': 'li-api-v1',
    'LI-AppLoja': 'li-app-loja',
    'LI-AppConciliacao': 'li-app-conciliacao'
}


# CONFIGURACAO PARA SLACK
#########################
USE_SLACK = True
TEST_CHANNEL = "#teste_automacao"

# CONFIGURACAO PARA DATADOG
###########################
USE_DATADOG = True

# CONFIGURACAO PARA GRAFANA
###########################
USE_GRAFANA = True
GRAFANA_MSG_URL = "http://services-int.awsli.com.br:8086/write?db=msgs"

# DICIONARIO DE DADOS
#####################

CONFIG_DICT = {
    "project_path": None,
    "docker_compose_path": None
}

if USE_AWS:
    CONFIG_DICT['aws_key'] = None
    CONFIG_DICT['aws_secret'] = None
    CONFIG_DICT['aws_region'] = None
    CONFIG_DICT['aws_account'] = None

if USE_SLACK:
    CONFIG_DICT['slack_url'] = None
    CONFIG_DICT['slack_channel'] = None
    CONFIG_DICT['slack_icon'] = None
    CONFIG_DICT['slack_user'] = None

if USE_DATADOG:
    CONFIG_DICT['datadog_api_key'] = None
    CONFIG_DICT['datadog_app_key'] = None



#!/usr/bin/env python
# coding: utf-8
"""Ferramenta Loja Integrada.
Para mais detalhes digite 'li help'

Usage:
    li bash     [<app>]
    li build    [<app>] [--no-cache]
    li config   [--clone-only]
    li debug    [<app>]
    li deploy
    li help     [<option>]
    li list     [<libs>...]
    li rebuild  [-y | --yes]
    li run      [<app>] [<command> ...]
    li telnet   [<app>] (<port>)
    li test     [<app>] [--using=(django|nose|pytest|behave)] [--rds]
    li tunnel   [<subdomain>] [<app>]
    li update   [-y | --yes] [--production | --staging]

Options:
    --help          Mostra esta tela
    --django        Roda o teste unitario do Django. (Padrao: Unittest.)
    --rds           Nos testes usar o RDS da Amazon
    --no-cache      Na build nao utilizar o cache
    <command>       Rode um comando para o run do container
    -y --yes        Confirma automaticamente
    <libs>          Mostrar a versão das livrarias solicitadas
    --production    Muda para a branch mais estável (ex. 'production')
    --staging       Altera as branchs para staging/beta durante o update
    <subdomain>     O subdominio para o tunel reverso, via ngrok

"""
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import json
from docopt import docopt
from tools import __version__
from tools import docker
from tools import settings
from tools.config import get_config_data, run_update
from tools.deploy import run_deploy
from tools.help import get_help
from tools.lists import show_list
from tools.tunnel import run_ngrok
from tools.utils import bcolors, confirma, run_command
from tools.version import show_version_warning


def check_vpn():
    # Checa se a VPN está ativa
    if settings.CHECK_VPN:
        ret_tun = run_command(
            get_stdout=True,
            command_list=[
                {
                    'command': 'ifconfig | grep tun',
                    'run_stdout': False
                }
            ]
        )

        if not ret_tun:
            print("\n{}{}ERRO:{} VPN não encontrada.".format(bcolors.BOLD,bcolors.FAIL,bcolors.ENDC))
            print("Por favor, ative a VPN e tente novamente.")
            return False

    return True


def main():
    """Faz o Parse dos Comandos"""
    arguments = docopt(__doc__, version=__version__)

    if not arguments['help']:
        print("Para ajuda digite: li help")

    #
    # CONFIG
    #
    if arguments['config'] is True and check_vpn():
        clone_only = arguments['--clone-only']
        data = get_config_data()
        if data and not clone_only:
            print("Configuração Atual:")
            print(json.dumps(data, indent=2))
        if clone_only:
            resposta = "S"
        else:
            resposta = confirma(u"Deseja rodar a configuração?")
        if resposta == "S":
            data = get_config_data(
                start_over=True,
                clone_only=clone_only
            )
        return True
    #
    # DEPLOY
    #
    if arguments['deploy'] is True and check_vpn():
        ret = run_deploy()
        return ret
    #
    # DEBUG
    #
    if arguments['debug'] is True:
        ret = docker.run_debug(
            application=arguments['<app>']
        )
        return ret
    #
    # TELNET
    #
    if arguments['telnet'] is True:
        ret = docker.run_telnet(
            application=arguments['<app>'],
            port=arguments['<port>']
        )
        return ret
    #
    # BASH
    #
    if arguments['bash'] is True:
        ret = docker.run_bash(
            application=arguments['<app>']
        )
        return ret
    #
    # TEST
    #
    if arguments['test'] is True:
        ret = docker.run_test(
            application=arguments['<app>'],
            using=arguments['--using'],
            rds=arguments['--rds']
        )
        return ret
    #
    # RUN APP
    #
    if arguments['run'] is True:
        ret = docker.run_runapp(
            application=arguments['<app>'],
            action='up' if not arguments['<command>'] else 'exec',
            arg=arguments['<command>']
        )
        return ret
    #
    # BUILD ADD
    #
    if arguments['build'] is True:
        ret = docker.run_runapp(
            application=arguments['<app>'],
            action='build',
            opt="--no-cache" if arguments['--no-cache'] else None
        )
        return ret
    #
    # UPDATE
    #
    if arguments['update'] is True:
        ret = run_update(
            no_confirm=arguments['--yes'],
            stable=arguments['--production'],
            staging=arguments['--staging'])
        return ret
    #
    # REBUILD
    #
    if arguments['rebuild'] is True:
        ret = docker.rebuild_docker(no_confirm=arguments['--yes'])
        return ret
    #
    # HELP
    #
    if arguments['help'] is True:
        ret = get_help(app=arguments['<option>'])
        return ret
    #
    # LIST
    #
    if arguments['list'] is True:
        ret = show_list(libs=arguments['<libs>'])
        return ret
    #
    # TUNNEL
    # 
    if arguments['tunnel'] is True and check_vpn():
        ret = run_ngrok(
            subdomain=arguments['<subdomain>'],
            app=arguments['<app>']
        )
        return ret


def start():
    print(
        "\033[94m\033[1m\n\n************************\n\n"
        "{cmd}-Tools v{version}\n\n"
        "************************\n\033[0m".format(
            cmd=settings.TERMINAL_CMD.upper(), version=__version__)
    )
    show_version_warning()
    retorno = main()
    if retorno:
        print('\n')
    else:
        print("\nOperação finalizada.\n")


if __name__ == "__main__":
    start()



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals, with_statement, nested_scopes
import os
import platform
from git import Repo
from os.path import expanduser
from tools import settings
from tools.messages import notify
from tools.utils import run_command, confirma


def get_config_data(
        clone_only=False,
        filename=settings.CONFIG_FILE,
        start_over=False):
    # Verifica se a configuracao existe
    # Caso nao exista perguntar
    config = settings.CONFIG_DICT
    basepath = expanduser("~")
    filepath = os.path.join(basepath, ".{}".format(filename))

    # Checa:
    # 1. arquivo de configuracao existe,
    # 2. arquivo está completo
    # 3. a variavel de ambiente está na memoria
    # 4. a pasta das aplicações existe

    ret = True

    if not os.path.exists(filepath):
        ret = False
    else:
        with open(filepath, 'r') as file:
            for line in file:
                if "=" in line:
                    key = line.split("=")[0].lower()
                    value = line.split("=")[1].rstrip()
                    config[key] = value

        for key in config:
            if not config.get(key):
                ret = False

        if not os.environ.get(settings.ENV_NAME):
            ret = False
        elif not os.path.exists(os.environ.get(settings.ENV_NAME)):
            ret = False

    project_path = config['project_path']
    if not project_path:
        return False
    if ret and not start_over:
        return config
    elif not clone_only:
        path_message = None
        print("\n\033[93m>> Configuração")
        print("***************\033[0m")
        config_list = [
            '.bash_profile',
            '.bashrc',
            '.zshrc',
            '.profile'
        ]
        if start_over:
            resp = confirma("Deseja configurar as chaves")
        else:
            resp = "S"
        if resp == "S":
            with open(filepath, 'w') as file:
                for key in config:
                    if key == "docker_compose_path" and not config.get(key):
                        continue
                    if config.get(key):
                        ask = "Informe {} [{}]: ".format(
                            key.upper(), config.get(key))
                    else:
                        ask = "Informe {}: ".format(key.upper())

                    resposta_ok = False
                    while not resposta_ok:
                        try:
                            value = str(raw_input(ask))
                            if not value and config[key]:
                                file.write(
                                    "{}={}\n".format(
                                        key.upper(), config[key]))
                                resposta_ok = True
                            if value:
                                config[key] = value
                                file.write(
                                    "{}={}\n".format(
                                        key.upper(), value))
                                resposta_ok = True
                        except KeyboardInterrupt:
                            print("\nOperação interrompida")
                            return False
                        except:
                            print('erro')
                            pass
            # Grava arquivo de credenciais da Amazon
            if settings.USE_AWS:
                aws_folder = os.path.join(basepath, ".aws")
                if not os.path.exists(aws_folder):
                    os.makedirs(aws_folder)
                with open(os.path.join(aws_folder, "config"), 'w') as file:
                    file.write("[config]\n")
                    file.write('region = {}\n'.format(config['aws_region']))

                with open(os.path.join(aws_folder, "credentials"), 'w') as file:
                    file.write('[default]\n')
                    file.write(
                        'aws_access_key_id = {}\n'.format(
                            config['aws_key']))
                    file.write(
                        'aws_secret_access_key = {}\n'.format(
                            config['aws_secret']))

        # Grava a variavel de ambiente nos arquivos de configuracao
        print("\n")
        one_file_ok = False
        for filename in config_list:
            export_found = False
            try:
                profile_path = os.path.join(basepath, filename)
                with open(profile_path) as file:
                    for line in file:
                        if settings.ENV_NAME in line:
                            export_found = True
                            one_file_ok = True
                    if not export_found:
                        ret = run_command(
                            title=None,
                            command_list=[
                                {
                                    'command': "echo export {}='{}' >> {}".format(
                                        settings.ENV_NAME,
                                        project_path,
                                        profile_path),
                                    'run_stdout': False}])
                        if ret:
                            print(
                                "Adicionando {} no arquivo {}".format(
                                    settings.ENV_NAME, filename))
                            export_found = True
                            one_file_ok = True
                    else:
                        export_found = True
                        one_file_ok = True
            except:
                pass
            finally:
                run_command(
                    title=None,
                    command_list=[
                        {
                            'command': "export {}='{}'".format(
                                settings.ENV_NAME,
                                project_path),
                            'run_stdout': False
                        }
                    ]
                )
        if not one_file_ok:
            path_message = "Certifique-se que o {} esteja no seu arquivo .profile, .bashrc ou .zshrc correspondente".format(
                settings.ENV_NAME)

    # Clona os repositorios LI
    clone_action = False
    if clone_only:
        resp = "S"
    else:
        resp = confirma("\nDeseja clonar os Repositórios")
    if resp == "S":
        clone_action = True
        if not os.path.exists(project_path):
            os.makedirs(project_path)
        run_command(
            title="Clonando Repositorios",
            command_list=[
                {
                    'command': "git config --global credential.helper 'cache --timeout=3600'",
                    'run_stdout': False}])

        # Baixa APPLICATIONS
        baixa_repositorios(config)

    # Confirma o caminho do docker-compose.yml
    if not config.get('docker_compose_path') and not clone_only:
        ret = run_command(
            title="Localizando arquivo docker-compose.yml",
            get_stdout=True,
            command_list=[
                {
                    'command': "locate docker-compose.yml",
                    'run_stdout': False
                }
            ]
        )
        if ret:
            paths_found = ret.split('\n')
            if paths_found[-1] == '':
                paths_found.pop(-1)
            if len(paths_found) == 1:
                config['docker_compose_path'] = paths_found[
                    0].replace('docker-compose.yml', '')
            elif paths_found:
                print(
                    u"Informe a localização do arquivo 'docker-compose.yml' do Projeto")
                print(u"(A localização padrão é: '{}/{}')\n".format(
                    project_path,
                    settings.DOCKER_REPO_NAME
                ))
                print("Os caminhos encontrados foram:")
                for num, path in enumerate(paths_found):
                    print("{}. {}".format(num + 1, path))
                resposta_ok = False
                print("\n")
                while not resposta_ok:
                    try:
                        rep = raw_input(
                            "Selecione o caminho: (1-{}): ".format(num + 1))
                        if rep and int(rep) in xrange(1, num + 1):
                            resposta_ok = True
                    except KeyboardInterrupt:
                        print("Operação interrompida\n")
                        return False
                    except:
                        pass
                config['docker_compose_path'] = paths_found[
                    int(rep) - 1].replace('docker-compose.yml', '')
        else:
            resposta_ok = False
            while not resposta_ok:
                try:
                    rep = raw_input(
                        "Informe o caminho do arquivo docker-compose.yml: ")
                    if os.path.exists(
                        os.path.join(
                            rep,
                            "docker-compose.yml")):
                        resposta_ok = True
                        config['docker_compose_path'] = rep
                except KeyboardInterrupt:
                    print("Operação interrompida\n")
                    return False
                except:
                    pass

        if config.get('docker_compose_path'):
            print('Arquivo encontrado!')
            with open(filepath, 'a') as file:
                file.write(
                    "{}={}\n".format(
                        "DOCKER_COMPOSE_PATH",
                        config.get('docker_compose_path')
                    ))

    if clone_only:
        notify(msg="Configuração finalizada.")
        return True

    print("\n\n\nConfiguração concluída.")
    print("Feche esta tela e abra um novo terminal antes de continuar.")
    print("Para trabalhar com os repositórios certifique-se que:")
    print("* O docker e o docker-compose estejam instalados.")
    if platform.system() == "Windows":
        print("* (Windows) A variável de ambiente {} esteja configurada.".format(settings.ENV_NAME))
        print("* (Windows) Rode o comando 'aws configure'")
    elif path_message:
        print("* {}".format(path_message))
    print("* O comando 'aws configure' tenha sido rodado no repositório, antes do deploy.")
    print("* O comando 'eb init' tenha sido rodado no repositório, antes do deploy.")
    if clone_action:
        notify(msg="Configuração finalizada.")
    return False


def run_update(no_confirm, stable, staging):
    data = get_config_data()

    if not data:
        return False

    print("\n\033[1m\033[93m>> Atualizar Repositórios")
    print("****************************\033[0m")

    if no_confirm:
        resp = "S"
    else:
        resp = confirma(
            u"Este comando atualiza todos os Repositórios\n"
            "que estejam nas branchs 'production', 'staging'\n"
            "'release', 'beta' ou 'master'.\n"
            "Além disso, baixa novos repositórios que ainda\n"
            "não estejam na pasta do projeto.\n"
            "Deseja continuar")

    if resp == "S":
        # Iterar todos os repositórios
        # Checar em que branch está
        # Se tiver em production ou master, dar o git pull
        for app in settings.APPLICATIONS:
            app_name = app[0]
            stable_branch = app[1][-1]
            caminho = os.path.join(data['project_path'], app_name)
            if os.path.exists(caminho):
                repo = Repo(
                    os.path.join(data['project_path'], app_name)
                )
                # 1. Checa se o remote existe e é válido
                remote_url_list = repo.remotes
                origin = [
                    obj
                    for obj in remote_url_list
                    if obj.name == 'origin'
                ]
                if origin:
                    origin = origin[0]
                remote_url = "{}{}.git".format(
                    settings.VCS_BASE_URL,
                    app_name.lower())
                if origin:
                    if origin.url != remote_url:
                        if no_confirm:
                            resp = "S"
                        else:
                            resp = confirma(
                                u"O repositório '{}' está com o endereço\n"
                                "remoto: {}.\nDeseja trocar".format(
                                    app_name, origin.url))
                        if resp == "S":
                            origin.set_url(
                                new_url=remote_url
                            )
                else:
                    repo.create_remote(name="origin", url=remote_url)

                # 2. Se tiver a opção 'stable'
                # mudar para a branch mais estável
                if stable:
                    os.chdir(caminho)
                    run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git checkout {branch}'.format(
                                    branch=stable_branch
                                ),
                                'run_stdout': False
                            }
                        ]
                    )

                # Checa Staging
                if staging:
                    stage_branch = None
                    if 'staging' in app[1]:
                        stage_branch = 'staging'
                    if 'beta' in app[1]:
                        stage_branch = 'beta'

                    if not stage_branch:
                        stage_branch = stable_branch

                    os.chdir(caminho)
                    run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git checkout {branch}'.format(
                                    branch=stage_branch
                                ),
                                'run_stdout': False
                            }
                        ]
                    )

                

                # 3. Checa se o repositorio é production/master
                # e faz o update
                branch = repo.active_branch.name
                if branch in ['production', 'staging', 'beta', 'release', 'master']:
                    print(
                        "\n\033[1m\033[94mAtualizando '{}/{}'\033[0m".format(app_name, branch))
                    os.chdir(caminho)
                    run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git remote update && git fetch && git pull --all',
                                'run_stdout': False
                            }
                        ]
                    )
                else:
                    print(
                        "\n\033[1m\033[93mRepositório '{}' ignorado. Branch: {}\033[0m".format(
                            app_name, branch))
        # Baixa os repositorios faltantes
        baixa_repositorios(data)
        notify(msg="Update dos projetos finalizado.")


def baixa_repositorios(data):
    for app, branch_list in settings.APPLICATIONS:
        if os.path.exists(os.path.join(data['project_path'], app)):
            continue
        print("\n\033[1m\033[94mBaixando '{}'\033[0m".format(app))
        first_branch = True
        for branch in branch_list:
            github_url = "{}{}.git".format(
                settings.VCS_BASE_URL,
                app.lower())
            if first_branch:
                ret = run_command(
                    title=None,
                    command_list=[
                        {
                            'command': 'git clone -b {branch} {url} "{dir}"'.format(
                                branch=branch,
                                url=github_url,
                                dir=os.path.join(
                                    data['project_path'],
                                    app)),
                            'run_stdout': False}])
                first_branch = False
            else:
                if ret:
                    os.chdir(os.path.join(data['project_path'], app))
                    run_command(
                        title=None,
                        command_list=[
                            {
                                'command': 'git checkout -b {branch} remotes/origin/{branch}'.format(
                                    branch=branch
                                ),
                                'run_stdout': False
                            }
                        ]
                    )
